package com.wyattjmiller.exam2
import android.content.ContentValues
import android.content.Context
import android.database.sqlite.SQLiteDatabase
import android.database.sqlite.SQLiteOpenHelper
import java.util.*

class FoodDatabase(context: Context?) : SQLiteOpenHelper(context, DATABASE_NAME, null, VERSION) {
    override fun onCreate(db: SQLiteDatabase) {
        db.execSQL(
            "create table " + FoodTable.TABLE + " (" +
                    FoodTable.COL_ID + " integer primary key autoincrement, " +
                    FoodTable.COL_RESTAURANT + " text, " +
                    FoodTable.COL_WEBSITE + " text, " +
                    FoodTable.COL_FAVORITEFOOD + " text)"
        )
    }

    override fun onUpgrade(db: SQLiteDatabase, oldVersion: Int, newVersion: Int) {
        db.execSQL("drop table if exists " + FoodTable.TABLE)
        onCreate(db)
    }

    override fun onOpen(db: SQLiteDatabase) {
        super.onOpen(db)
        if (!db.isReadOnly) {
            // Enable foreign key constraints
            db.setForeignKeyConstraintsEnabled(true)
        }
    }

    fun getFoods(): MutableList<Food> {
        val foods: MutableList<Food> = ArrayList<Food>()
        val db = this.readableDatabase
        val sql =
            "select * from " + FoodTable.TABLE
        val cursor = db.rawQuery(sql, null)

        if (cursor.moveToFirst()) {
            do {
                val food = Food()
                food.id = cursor.getInt(0)
                food.foodRestaurant = cursor.getString(1)
                food.foodWebsite = cursor.getString(2)
                food.foodFavorite = cursor.getString(3)
                foods.add(food)
            } while (cursor.moveToNext())
        }

        cursor.close()
        return foods
    }

    fun getFood(courseid: Int): Food {
        var food = Food()
        val db = this.readableDatabase
        val sql =
            "select * from " + FoodTable.TABLE +
                    " where " + FoodTable.COL_ID + " = ?"
        val cursor = db.rawQuery(sql, arrayOf(java.lang.Float.toString(courseid.toFloat())))

        if (cursor.moveToFirst()) {
            food.id = cursor.getInt(0)
            food.foodRestaurant = cursor.getString(1)
            food.foodWebsite = cursor.getString(2)
            food.foodFavorite = cursor.getString(3)
        }

        cursor.close()
        return food
    }

    fun addFood(food: Food): Boolean {
        val db = writableDatabase
        val values = ContentValues()
        values.put(FoodTable.COL_RESTAURANT, food.foodRestaurant)
        values.put(FoodTable.COL_WEBSITE, food.foodWebsite)
        values.put(FoodTable.COL_FAVORITEFOOD, food.foodFavorite)
        val id = db.insert(FoodTable.TABLE, null, values)
        food.id = id.toInt()
        return id != -1L
    }

    fun updateFood(food: Food) {
        val db = writableDatabase
        val values = ContentValues()
        values.put(FoodTable.COL_RESTAURANT, food.foodRestaurant)
        values.put(FoodTable.COL_WEBSITE, food.foodWebsite)
        values.put(FoodTable.COL_FAVORITEFOOD, food.foodFavorite)
        db.update(FoodTable.TABLE, values, FoodTable.COL_ID + " = " + food.id, null)
    }

    fun deleteFood(food: Food) {
        val db = writableDatabase
        db.delete(FoodTable.TABLE, FoodTable.COL_RESTAURANT + " = ?", arrayOf(food.foodRestaurant))
    }

    companion object {
        private const val DATABASE_NAME = "courses.db"
        private const val VERSION = 1
        private var mFoodDb: FoodDatabase? = null

        fun getInstance(context: Context?): FoodDatabase {
            if (mFoodDb == null) {
                mFoodDb = FoodDatabase(context)
            }
            return mFoodDb!!
        }
    }
}