package com.wyattjmiller.exam2

import android.app.Dialog
import android.content.Context
import android.os.Bundle
import android.widget.Button
import android.widget.EditText
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.fragment.app.DialogFragment

class FoodDialogFragment : DialogFragment() {
    private var mTitleTextView: TextView? = null
    private lateinit var mRestaurantTextView: TextView
    private lateinit var mWebsiteTextView: TextView
    private lateinit var mFavoriteFoodTextView: TextView
    private lateinit var mRestaurantEditText: EditText
    private lateinit var mWebsiteEditText: EditText
    private lateinit var mFavoriteFoodEditText: EditText
    private lateinit var mConfirmButton: Button
    private var mFood: Food = Food()
    private lateinit var mFoodDb: FoodDatabase
    private lateinit var mFoodContext: FoodContext

    interface onCourseEnteredListener {
        fun onCourseEntered(food: Food)
    }

    private var mListener: onCourseEnteredListener? = null


    override fun onAttach(context: Context) {
        super.onAttach(context)
        mListener = context as onCourseEnteredListener
    }

    override fun onCreateDialog(savedInstanceState: Bundle?) : Dialog {
        val builder = AlertDialog.Builder(this.activity!!)
        val dialogView = activity!!.layoutInflater.inflate(R.layout.activity_main_edit, null)

        mTitleTextView = dialogView.findViewById(R.id.textViewTitle)

        mRestaurantTextView = dialogView.findViewById(R.id.textViewRestaurant)
        mWebsiteTextView = dialogView.findViewById(R.id.textViewWebiste)
        mFavoriteFoodTextView = dialogView.findViewById(R.id.textViewFavoriteFood)

        mRestaurantEditText = dialogView.findViewById(R.id.editTextRestaurant)
        mWebsiteEditText = dialogView.findViewById(R.id.editTextWebsite)
        mFavoriteFoodEditText = dialogView.findViewById(R.id.editTextFavoriteFood)

        mConfirmButton = dialogView.findViewById(R.id.buttonConfirm)

        when (mFoodContext) {
            FoodContext.ADD -> {
                mTitleTextView?.text = getString(R.string.food_title_add)
                mConfirmButton.text = getString(R.string.food_add)
            }
            FoodContext.EDIT -> {
                val food = mFoodDb.getFood(mFood.id!!)

                mTitleTextView?.text = getString(R.string.food_title_edit)
                mConfirmButton.text = getString(R.string.food_update)

                mRestaurantEditText.setText(food.foodRestaurant)
                mWebsiteEditText.setText(food.foodWebsite)
                mFavoriteFoodEditText.setText(food.foodFavorite)
            }
            FoodContext.VIEW -> {
                val food = mFoodDb.getFood(mFood.id!!)

                mTitleTextView?.text = getString(R.string.food_title_view)
                mConfirmButton.text = getString(R.string.food_view)

                mRestaurantEditText.setText(food.foodRestaurant)
                mWebsiteEditText.setText(food.foodWebsite)
                mFavoriteFoodEditText.setText(food.foodFavorite)

                mRestaurantEditText.setTextIsSelectable(false)
                mWebsiteEditText.setTextIsSelectable(false)
                mFavoriteFoodEditText.setTextIsSelectable(false)
            }
        }

        builder.setView(dialogView)

        mConfirmButton.setOnClickListener {
            when (mFoodContext) {
                FoodContext.ADD -> {
                    mFood.foodRestaurant = mRestaurantEditText.text.toString()
                    mFood.foodWebsite = mWebsiteEditText.text.toString()
                    mFood.foodFavorite = mFavoriteFoodEditText.text.toString()

                    if (validityCheck()) {
                        mListener?.onCourseEntered(mFood)
                        dismiss()
                    }
                }
                FoodContext.EDIT -> {
                    mFood.foodRestaurant = mRestaurantEditText.text.toString()
                    mFood.foodWebsite = mWebsiteEditText.text.toString()
                    mFood.foodFavorite = mFavoriteFoodEditText.text.toString()

                    if (validityCheck()) {
                        mListener?.onCourseEntered(mFood)
                        dismiss()
                    }
                }
                FoodContext.VIEW -> {
//                    mRestaurantEditText
                    dismiss()
                }
            }
        }

        return builder.create()
    }

    private fun validityCheck(): Boolean {
        val regex = "^((https?|ftp|smtp):\\/\\/)?(www.)?[a-z0-9]+\\.[a-z]+(\\/[a-zA-Z0-9#]+\\/?)*\$".toRegex()
        return if (mRestaurantEditText?.text.toString() == "" ||
            mWebsiteEditText?.text.toString() == "" ||
            mFavoriteFoodEditText?.text.toString() == "") {
            Toast.makeText(this.activity!!, "Please fill in all of the fields", Toast.LENGTH_LONG).show()
            false
        } else {
            if (regex.matches(mWebsiteEditText?.text.toString())) true else {
                Toast.makeText(this.activity!!, "Is this a URL?", Toast.LENGTH_LONG).show()
                false
            }
        }
    }

    fun getContext(context: FoodContext, database: FoodDatabase) {
        mFoodContext = context
        mFoodDb = database
    }

    fun getFoodId(foodId: Int) {
        mFood?.id = foodId
    }

    companion object {



    }
}