package com.wyattjmiller.exam2

import android.content.Intent
import android.content.SharedPreferences
import android.os.Bundle
import android.os.Handler
import android.preference.PreferenceManager
import android.view.View
import android.widget.Button
import android.widget.NumberPicker
import android.widget.ProgressBar
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity
import java.text.DecimalFormat


class TimerActivity : AppCompatActivity() {
    private var mHoursPicker: NumberPicker? = null
    private var mMinutesPicker: NumberPicker? = null
    private var mSecondsPicker: NumberPicker? = null
    private var mStartButton: Button? = null
    private var mPauseButton: Button? = null
    private var mCancelButton: Button? = null
    private var mProgressBar: ProgressBar? = null
    private var mTimeLeftTextView: TextView? = null
    private var mHandler: Handler? = null
    private var mTimerModel: Timer? = null
    private var mSharedPrefs: SharedPreferences? = null
    private var mDarkTheme: Boolean = false

    override fun onCreate(savedInstanceState: Bundle?) {
        mSharedPrefs = PreferenceManager.getDefaultSharedPreferences(this)
        mDarkTheme = mSharedPrefs!!.getBoolean(SettingsFragment.PREFERENCE_THEME, false)

        if (mDarkTheme) setTheme(R.style.DarkTheme)

        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_timer)

        // Initially hide the timer and progress bar
        mTimeLeftTextView = findViewById(R.id.timeLeft)
        mTimeLeftTextView?.visibility = View.INVISIBLE
        mProgressBar = findViewById(R.id.progressBar)
        mProgressBar?.visibility = View.INVISIBLE
        mStartButton = findViewById(R.id.startButton)
        mPauseButton = findViewById(R.id.pauseButton)
        mCancelButton = findViewById(R.id.cancelButton)

        // Hide pause and cancel buttons until the timer starts
        mPauseButton?.visibility = View.GONE
        mCancelButton?.visibility = View.GONE

        // Show 2 digits in NumberPickers
        val numFormat =
            NumberPicker.Formatter { i -> DecimalFormat("00").format(i.toLong()) }

        // Set min and max values for all NumberPickers
        mHoursPicker = findViewById(R.id.hoursPicker)
        mHoursPicker?.minValue = 0
        mHoursPicker?.maxValue = 99
        mHoursPicker?.setFormatter(numFormat)
        mMinutesPicker = findViewById(R.id.minutesPicker)
        mMinutesPicker?.minValue = 0
        mMinutesPicker?.maxValue = 59
        mMinutesPicker?.setFormatter(numFormat)
        mSecondsPicker = findViewById(R.id.secondsPicker)
        mSecondsPicker?.minValue = 0
        mSecondsPicker?.maxValue = 59
        mSecondsPicker?.setFormatter(numFormat)
        mTimerModel = Timer()

        // Instantiate default Handler so Runnables can be posted to UI message queue
        mHandler = Handler()
    }

    override fun onStop() {
        super.onStop()

        // Start the service if the timer is running
        if (mTimerModel!!.isRunning) {
            val intent = Intent(this, TimerIntentService::class.java)
            intent.putExtra(TimerIntentService.EXTRA_MILLIS_LEFT, mTimerModel?.remainingMilliseconds)
            startService(intent)
        }
    }

    fun startButtonClick(view: View?) {
        // Get values from NumberPickers
        val hours = mHoursPicker!!.value
        val minutes = mMinutesPicker!!.value
        val seconds = mSecondsPicker!!.value
        if (hours + minutes + seconds > 0) {
            // Show progress
            mTimeLeftTextView!!.visibility = View.VISIBLE
            mProgressBar!!.progress = 0
            mProgressBar!!.visibility = View.VISIBLE

            // Show only Pause and Cancel buttons
            mStartButton!!.visibility = View.GONE
            mPauseButton!!.visibility = View.VISIBLE
            mPauseButton!!.setText(R.string.pause)
            mCancelButton!!.visibility = View.VISIBLE

            // Start the model
            mTimerModel?.start(hours, minutes, seconds)

            // Start sending Runnables to message queue
            mHandler!!.post(mUpdateTimerRunnable)
        }
    }

    fun pauseButtonClick(view: View?) {
        if (mTimerModel?.isRunning!!) {
            // Pause and change to resume button
            mTimerModel?.pause()
            mHandler!!.removeCallbacks(mUpdateTimerRunnable)
            mPauseButton!!.setText(R.string.resume)
        } else {
            // Resume and change to pause button
            mTimerModel?.resume()
            mHandler!!.post(mUpdateTimerRunnable)
            mPauseButton!!.setText(R.string.pause)
        }
    }

    fun cancelButtonClick(view: View?) {
        mTimeLeftTextView!!.visibility = View.INVISIBLE
        mProgressBar!!.visibility = View.INVISIBLE
        timerCompleted()
    }

    private fun timerCompleted() {
        mTimerModel?.stop()

        // Remove any remaining Runnables that may reside in UI message queue
        mHandler!!.removeCallbacks(mUpdateTimerRunnable)

        // Show only the start button
        mStartButton!!.visibility = View.VISIBLE
        mPauseButton!!.visibility = View.GONE
        mCancelButton!!.visibility = View.GONE
    }

    private val mUpdateTimerRunnable: Runnable = object : Runnable {
        override fun run() {

            // Update UI to show remaining time and progress
            mTimeLeftTextView?.text = mTimerModel.toString()
            val progress: Int = mTimerModel!!.progressPercent
            mProgressBar!!.progress = progress

            // Only post Runnable if more time remains
            if (progress == 100) {
                timerCompleted()
            } else {
                mHandler!!.postDelayed(this, 200)
            }
        }
    }
}