package com.wyattjmiller.exam2

import android.app.IntentService
import android.app.Notification
import android.app.NotificationChannel
import android.app.NotificationManager
import android.content.Intent
import android.os.Build
import androidx.core.app.NotificationCompat
import androidx.core.app.NotificationManagerCompat

class TimerIntentService : IntentService("TimerIntentService") {
    override fun onHandleIntent(intent: Intent?) {

        // Get millis from the activity and start a new TimerModel
        val millisLeft = intent!!.getLongExtra(EXTRA_MILLIS_LEFT, 0)
        val timerModel = Timer()
        timerModel.start(millisLeft)

        // Create notification channel
        createTimerNotificationChannel()
        while (timerModel.isRunning) {
            try {
                createTimerNotification(timerModel.toString())
                Thread.sleep(1000)
                if (timerModel.remainingMilliseconds === 0L) {
                    timerModel.stop()
                    createTimerNotification("Timer is finished!")
                }
            } catch (e: InterruptedException) {
            }
        }
    }

    private fun createTimerNotificationChannel() {
        if (Build.VERSION.SDK_INT < 26) return
        val name: CharSequence = getString(R.string.channel_description_time)
        val description = getString(R.string.channel_name_time)
        val importance = NotificationManager.IMPORTANCE_LOW
        val channel = NotificationChannel(CHANNEL_ID_TIMER, name, importance)
        channel.description = description

        val notificationManager = getSystemService(NotificationManager::class.java)
        notificationManager.createNotificationChannel(channel)
    }


    private fun createTimerNotification(text: String) {
        val notification: Notification = NotificationCompat.Builder(this, CHANNEL_ID_TIMER)
            .setSmallIcon(R.drawable.timer)
            .setContentTitle(getString(R.string.app_name))
            .setContentText(text)
            .setPriority(NotificationCompat.PRIORITY_LOW)
            .build()

        val notificationManager = NotificationManagerCompat.from(this)
        notificationManager.notify(NOTIFICATION_ID, notification)
    }

    companion object {
        const val EXTRA_MILLIS_LEFT = "com.wyattjmiller.exam2.extra.EXTRA_MILLIS_LEFT"
        private const val CHANNEL_ID_TIMER = "channel_timer"
        private const val NOTIFICATION_ID = 0
    }
}
